/** ============================================================================
 *  @file   _notify.c
 *
 *  @path   $(DSPLINK)/gpp/src/api/WinCE/
 *
 *  @desc   Implemetation of NOTIFY.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>
#include <linkcfgdefs.h>
#include <procdefs.h>

/*  ----------------------------------- Trace & Debug                 */
#include <_trace.h>

/*  ----------------------------------- OSAL Header                   */
#include <drvdefs.h>
#include <drv_api.h>

/*  ----------------------------------- User API                      */
#include <notify.h>
#include <_notify.h>
#if defined (RINGIO_COMPONENT)
#include <ringio.h>
#include <_ringio.h>
#endif /* if defined (RINGIO_COMPONENT) */


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @name   COMPONENT_ID
 *
 *  @desc   Component ID of this module. That will be used by TRC macros.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_API_NOTIFY

/** ============================================================================
 *  @macro  THREAD_PRIORITY_NOTIFY
 *
 *  @desc   Priority of Notify Threads for Windows CE DSP BIOS LINK
 *  ============================================================================
 */
#define THREAD_PRIORITY_NOTIFY      111


/** ============================================================================
 *  @name   SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  \
            TRC_3PRINT (TRC_LEVEL7, \
                        "\nFailure: Status:[0x%x] File:[0x%x] Line:[%d]\n", \
                        status, FID_C_API_NOTIFY, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @macro  LINK_DRIVER
 *
 *  @desc   Name of the dsplink driver.
 *  ============================================================================
 */
#define LINK_DRIVER         _T("DBL1:")


/** ============================================================================
 *  @name   EVENT_Packet
 *
 *  @desc   Structure of Event Packet read from notify kernel-side.
 *
 *  @field  reserved
 *              Reserved.
 *  @field  eventNo
 *              Event Number.
 *  @field  data
 *              Data associated with event.
 *  @field  func
 *              User callback function.
 *  @field  param
 *              User callback argument.
 *  @field  isExit
 *              Tells whether to terminate notify thread.
 *  ============================================================================
 */
typedef struct EVENT_Packet_tag {
    Uint32       reserved [2] ;
    Uint32       eventNo      ;
    Uint32       data         ;
    FnNotifyCbck func         ;
    Pvoid        param        ;
#if defined(RINGIO_COMPONENT)
    Uint32       isRingio ;
#endif /* if defined(RINGIO_COMPONENT) */
} EVENT_Packet ;


/** ============================================================================
 *  @name   PROC_linkCfgPtr
 *
 *  @desc   Global pointer containing DSPLINK configuration information for the
 *          driver on the user-side.
 *  ============================================================================
 */
extern LINKCFG_Object * PROC_linkCfgPtr ;

/** ----------------------------------------------------------------------------
 *  @name   NOTIFY_state
 *
 *  @desc   Extern deckaration of the state object for the NOTIFY component
 *          containing all information required by it.
 *  ----------------------------------------------------------------------------
 */
extern NOTIFY_Object NOTIFY_state [MAX_DSPS] ;


/** ----------------------------------------------------------------------------
 *  @func   NOTIFY_eventWorker
 *
 *  @desc   This is the worker thread for polling on events.
 *
 *  @arg    arg
 *              Don't care.
 *
 *  @ret    None
 *
 *  @enter  None.
 *
 *  @leave  None
 *
 *  @see    None
 *  ----------------------------------------------------------------------------
 */
STATIC
DWORD
WINAPI
NOTIFY_eventWorker (IN     void * arg) ;


/** ============================================================================
 *  @func   _NOTIFY_init
 *
 *  @desc   This function initializes the NOTIFY component.
 *
 *  @modif  none
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_NOTIFY_init (IN ProcessorId dspId)
{
    DSP_STATUS          status = DSP_SOK ;
    Bool                bStatus = FALSE;
    CMD_Args            args ;
    LINKCFG_Dsp *       dspObj ;
    LINKCFG_LinkDrv *   linkDrv ;
    LINKCFG_Ips *       ipsObj ;
    Uint32              i ;
#if defined (RINGIO_COMPONENT)
    LINKCFG_RingIo *    ringIoObj ;
#endif /* #if defined (RINGIO_COMPONENT) */
    LINKCFG_DspConfig * dspConfig ;

    TRC_1ENTER ("_NOTIFY_init", dspId) ;

    DBC_Require (IS_VALID_PROCID (dspId)) ;

    dspConfig    = PROC_linkCfgPtr->dspConfigs [dspId] ;
    dspObj       = dspConfig->dspObject ;
    linkDrv   = &(dspConfig->linkDrvObjects [dspObj->linkDrvId]) ;

    NOTIFY_state [dspId].dspId = dspId ;

    for (i = 0 ; i < linkDrv->numIpsEntries ; i++) {
        ipsObj = &(dspConfig->ipsTables [linkDrv->ipsTableId][i]) ;
        NOTIFY_state [dspId].maxIpsEvents [i] = ipsObj->numIpsEvents ;
    }
#if defined (RINGIO_COMPONENT)
    ringIoObj = &(dspConfig->ringIoObjects [linkDrv->ringIoTableId]) ;
    NOTIFY_state [dspId].ringIoIpsId      = ringIoObj->ipsId ;
    NOTIFY_state [dspId].ringIoIpsEventNo = ringIoObj->ipsEventNo ;
#endif /* #if defined (RINGIO_COMPONENT) */

    args.apiArgs.notifyInitializeArgs.dspId = dspId ;

    status = DRV_INVOKE (DRV_handle, CMD_NOTIFY_INITIALIZE, &args) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }
    else {        
        /* Create the signal worker thread */
        NOTIFY_state [dspId].pData = (UINT32)CreateThread (NULL, 0,
                                                    (LPTHREAD_START_ROUTINE)NOTIFY_eventWorker, (LPVOID)DRV_handle,
                                                    CREATE_SUSPENDED, NULL);
        if ((HANDLE)NOTIFY_state [dspId].pData == NULL) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
        else {
            bStatus = CeSetThreadPriority ((HANDLE)NOTIFY_state [dspId].pData, THREAD_PRIORITY_NOTIFY );

            if (bStatus != TRUE) {
                TRC_0PRINT(TRC_LEVEL7,"LINK: _NOTIFY_init: CeSetThreadPriority() fail\r\n");
            }

            ResumeThread ((HANDLE)NOTIFY_state [dspId].pData);
        }        
    }

    TRC_1LEAVE ("_NOTIFY_init", status) ;

    return status ;
}

/** ============================================================================
 *  @func   _NOTIFY_exit
 *
 *  @desc   This function finalizes the NOTIFY component.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
_NOTIFY_exit (IN ProcessorId dspId)
{
    DSP_STATUS status = DSP_SOK ;
    CMD_Args   args             ;

    TRC_1ENTER ("_NOTIFY_exit", dspId) ;

    DBC_Require (IS_VALID_PROCID (dspId)) ;

    args.apiArgs.notifyFinalizeArgs.dspId = dspId ;

    status = DRV_INVOKE (DRV_handle, CMD_NOTIFY_FINALIZE, &args) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    if ((HANDLE)NOTIFY_state [dspId].pData != (Uint32) NULL) {
        WaitForSingleObject((HANDLE)NOTIFY_state [dspId].pData, INFINITE);
        CloseHandle((HANDLE)NOTIFY_state [dspId].pData);
        NOTIFY_state [dspId].pData = 0;
    }

    TRC_1LEAVE ("_NOTIFY_exit", status) ;

    return status ;
}


/** ----------------------------------------------------------------------------
 *  @func   NOTIFY_eventWorker
 *
 *  @desc   This is the worker thread which polls for events.
 *
 *  @modif  None.
 *  ----------------------------------------------------------------------------
 */
STATIC
DWORD
WINAPI
NOTIFY_eventWorker (IN     void * arg)
{
    DSP_STATUS      status = DSP_SOK ;
    DWORD           nRead  = 0 ;
    EVENT_Packet    packet ;
    HANDLE          drvHandle = 0;
    TCHAR           evtName[30] ;
    HANDLE          hEvent ;
    HANDLE          hWait[2];
    DWORD           waitResult;

    drvHandle = CreateFile( LINK_DRIVER, GENERIC_READ | GENERIC_WRITE, 0, NULL,
                                        OPEN_EXISTING, 0, 0 );
    if (drvHandle == INVALID_HANDLE_VALUE) {
        TRC_0PRINT(TRC_LEVEL7,"LINK: NOTIFY_eventWorker: DspLink Driver Open Failed\r\n");
        ExitThread (1) ;
    }
    while (DSP_SUCCEEDED (status)) {
            memset (&packet, 0, sizeof (EVENT_Packet)) ;
            if ( ReadFile (drvHandle, &packet, sizeof (EVENT_Packet), &nRead, NULL) ) {
                if ( nRead == sizeof (EVENT_Packet) ) {
                    if (packet.eventNo  == (Uint32) -1) {  

                        if (drvHandle)
                        {
                            CloseHandle(drvHandle);
                        }
        				return 0;
                    }
#if defined (RINGIO_COMPONENT)
                    if (packet.isRingio == 1) {
                        RingIO_notifyCallback (NOTIFY_state [0].ringIoIpsEventNo,
                                               packet.param,
                                               (Pvoid)packet.data) ;
                    }
                    else
#endif
					{
                        if (packet.func != NULL) {
                            packet.func (packet.eventNo,
                                         packet.param,
                                         (Pvoid)packet.data) ;
                        }
                    }
                    nRead = 0 ;
                }
            }
            else {
                status = DSP_EFAIL;
                TRC_0PRINT(TRC_LEVEL7,"LINK: NOTIFY_eventWorker: Failed to read from driver\r\n");
            }
    }
    return (0);
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
