/** ============================================================================
 *  @file   da8xxgem_hal.c
 *
 *  @path   $(DSPLINK)/gpp/src/arch/DA8XXGEM/
 *
 *  @desc   Power management module.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>
#include <linkcfgdefs.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>
#include <mem.h>

/*  ----------------------------------- Hardware Abstraction Layer  */
#include <da8xxgem_hal.h>
#if (DA8XXGEM_PHYINTERFACE == SHMEM_INTERFACE)
#include <da8xxgem_phy_shmem.h>
#endif /* if (DA8XXGEM_PHYINTERFACE == SHMEM_INTERFACE) */


#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_ARCH_HAL

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON      TRC_SetReason (status, FID_C_ARCH_HAL, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */

/** ============================================================================
 *  @name   LDRV_LinkCfgPtr
 *
 *  @desc   Global pointer containing DSPLINK configuration information for the
 *          driver.
 *  ============================================================================
 */
extern LINKCFG_Object * LDRV_LinkCfgPtr ;


/** ============================================================================
 *  @func   DA8XXGEM_halInitialize
 *
 *  @desc   Initializes the HAL object.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DA8XXGEM_halInit (IN     Pvoid * halObj,
                   IN     Pvoid   initParams)
{
    DSP_STATUS         status   = DSP_SOK ;
    DA8XXGEM_InitParams * params = (DA8XXGEM_InitParams *) initParams ;
    DA8XXGEM_HalObj * halObject ;
    LINKCFG_Dsp *        dspObj ;
    LINKCFG_DspConfig *  dspCfg ;

    TRC_2ENTER ("DA8XXGEM_halInit", halObj, initParams) ;

    DBC_Require (halObj != NULL) ;

    (void) initParams ;

    status = MEM_Alloc ((Pvoid) &halObject,
                        sizeof (DA8XXGEM_HalObj),
                        MEM_DEFAULT) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }
    else {
        *halObj = (Pvoid) halObject ;
        dspCfg    = LDRV_LinkCfgPtr->dspConfigs [params->dspId] ;
        dspObj    = dspCfg->dspObject ;

#if (DA8XXGEM_PHYINTERFACE == SHMEM_INTERFACE)
        halObject->interface = &DA8XXGEM_shmemInterface ;
#endif /* if (DA8XXGEM_PHYINTERFACE == SHMEM_INTERFACE) */
        status = halObject->interface->phyInit ((Pvoid) halObject) ;
        if (DSP_FAILED (status)) {
            FREE_PTR (halObject) ;
            SET_FAILURE_REASON ;
        }
        else {
            if(dspObj->arg4 == 0) {
                /* Unlocking the KICK registers once
                 * DSPLINK does not do lock and Unlock sequence
                 * as writing to kick registers  simultaneousely
                 * by both GPP and DSP causes failurs in sending interrupts.
                 */
                /* Unlock the kick0 area */
                *((volatile Uint32 *) (  halObject->baseCfgBus
                              + halObject->offsetSysModule
                              + OFFSET_KICK0 )) = KICK0_UNLOCKVAL ;

                /* Unlock the kick1 area */
                *((volatile Uint32 *) (  halObject->baseCfgBus
                              + halObject->offsetSysModule
                              + OFFSET_KICK1)) = KICK1_UNLOCKVAL ;
            }
        }
    }

    TRC_1LEAVE ("DA8XXGEM_halInit", status) ;

    return status ;
}


/** ============================================================================
 *  @func   DA8XXGEM_halExit
 *
 *  @desc   Finalizes the HAL object.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DA8XXGEM_halExit (IN     Pvoid * halObj)
{
    DSP_STATUS status   = DSP_SOK ;

    TRC_1ENTER ("DA8XXGEM_halExit", halObj) ;

    DBC_Require (halObj != NULL) ;

    FREE_PTR ((*halObj)) ;

    TRC_1LEAVE ("DA8XXGEM_halInit", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif
