/** ============================================================================
 *  @file   dm6437_hal_vlynq_map.c
 *
 *  @path   $(DSPLINK)/gpp/src/arch/DM6437/vlynq/
 *
 *  @desc   MAP control module.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- Hardware Abstraction Layer  */
#include <dm6437_hal.h>
#include <dm6437_hal_vlynq.h>
#include <dm6437_hal_vlynq_map.h>


#if defined (__cplusplus)
extern "C" {
#endif


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_ARCH_HAL_MAP


/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_ARCH_HAL_MAP, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @func   DM6437_halVlynqMapCtrl
 *
 *  @desc   Mapping controller.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
DM6437_halVlynqMapCtrl (IN      Pvoid           halObj,
                        IN      DSP_MapCtrlCmd  cmd,
                        IN      Uint32          dspBufAddr,
                        IN      Uint32          size,
                        OUT     Uint32 *        bufVirtPtr,
                        OUT     Uint32 *        bufPhysAddr,
                        OUT     Uint32 *        mappedSize)
{
    DSP_STATUS         status    = DSP_SOK ;
    DM6437_vlynqRegs * pvlynqRegs  = NULL  ;
    DM6437_HalObj *    halObject = (DM6437_HalObj *) halObj ;
    Uint32             availBytes          ;

    TRC_6ENTER ("DM6437_halVlynqMapCtrl",
                halObj,
                cmd,
                dspBufAddr,
                size,
                bufVirtPtr,
                bufPhysAddr) ;

    DBC_Require (NULL != halObject) ;
    DBC_Require (NULL != mappedSize) ;

    pvlynqRegs  = (DM6437_vlynqRegs *) (  halObject->vlynq.region3Addr
                                        + DM6437_PEERVLYNQREG_BASE) ;

    switch (cmd) {
        case DSP_MapCtrlCmd_Map:
        {
            availBytes = (  DM6437_VLYNQ_WINDOWSIZE
                          - pvlynqRegs->LOCAL_RAMS1
                          - pvlynqRegs->LOCAL_RAMS2
                          - pvlynqRegs->LOCAL_RAMS3) ;
            if ((availBytes > 0) && (size > 0)) {
                halObject->vlynq.prevUserAddr = pvlynqRegs->LOCAL_RAMO4 ;
                halObject->vlynq.prevUserSize = pvlynqRegs->LOCAL_RAMS4 ;
                pvlynqRegs->LOCAL_RAMO4 = dspBufAddr ;
                pvlynqRegs->LOCAL_RAMS4 = size ;
                *bufVirtPtr  = halObject->vlynq.region4Addr ;
                *bufPhysAddr = halObject->vlynq.region4Addr ;
                halObject->vlynq.region4Size = size ;
                if (availBytes >= size) {
                    *mappedSize = size ;
                 }
                 else {
                    *mappedSize = availBytes ;
                 }
            }
            else {
                status = DSP_EMEMORY ;
                SET_FAILURE_REASON ;
            }
        }
        break ;

        case DSP_MapCtrlCmd_Unmap:
        {
            pvlynqRegs->LOCAL_RAMO4 = halObject->vlynq.prevUserAddr ;
            pvlynqRegs->LOCAL_RAMS4 = halObject->vlynq.prevUserSize ;
        }
        break ;

        case DSP_MapCtrlCmd_SetShared:
        {
            pvlynqRegs->LOCAL_RAMO1 = dspBufAddr ;
            pvlynqRegs->LOCAL_RAMS1 = size ;
            halObject->vlynq.region1Size = size ;
            *bufVirtPtr  = halObject->vlynq.region1Addr ;
            *bufPhysAddr = halObject->vlynq.region1Addr ;
            /* Correct the regions */
            halObject->vlynq.region2Addr =  halObject->vlynq.region1Addr
                                          + halObject->vlynq.region1Size ;
            halObject->vlynq.region3Addr =  halObject->vlynq.region2Addr
                                          + halObject->vlynq.region2Size ;
            halObject->vlynq.region4Addr =  halObject->vlynq.region3Addr
                                          + halObject->vlynq.region3Size ;
            *mappedSize = size ;
        }
        break ;

        default:
        {
            /* Unsupported interrupt control command */
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
        break ;
    }

    TRC_1LEAVE ("DM6437_halVlynqIntCtrl", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif
