/** ============================================================================
 *  @file   coff_int.c
 *
 *  @path   $(DSPLINK)/gpp/src/gen/
 *
 *  @desc   Defines generic functions of COFF loader.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_bitops.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <kfile.h>

/*  ----------------------------------- Generic Functions           */
#include <coff_int.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID         ID_GEN_COFF

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define  SET_FAILURE_REASON TRC_SetReason (status, FID_C_GEN_COFF_INT, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @func   COFF_read8
 *
 *  @desc   Reads a Int8 from file.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Int8
COFF_read8 (IN KFileObject * fileObj)
{
    Int8 retVal = 0 ;

    TRC_1ENTER ("COFF_read8", fileObj) ;

    DBC_Require (fileObj != NULL) ;

    KFILE_Read ((Char8 *) &retVal, READ_REC_SIZE, sizeof (Int8), fileObj) ;

    TRC_0LEAVE ("COFF_read8") ;

    return retVal ;
}


/** ============================================================================
 *  @func   COFF_read16
 *
 *  @desc   Reads a Int16 from file.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Int16
COFF_read16 (IN KFileObject * fileObj, IN Bool swap)
{
    Int16 retVal ;

    TRC_2ENTER ("COFF_read16", fileObj, swap) ;

    DBC_Require (fileObj != NULL) ;

    KFILE_Read ((Char8 *) ((Pvoid) &retVal),
                READ_REC_SIZE,
                sizeof (Int16),
                fileObj) ;

    if (swap == TRUE) {
        retVal = BYTESWAP_WORD (retVal) ;
    }

    TRC_0LEAVE ("COFF_read16") ;

    return retVal ;
}


/** ============================================================================
 *  @func   COFF_read32
 *
 *  @desc   Reads a Int32 from file.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Int32
COFF_read32 (IN KFileObject * fileObj, IN Bool swap)
{
    Int32 retVal ;

    TRC_2ENTER ("COFF_read32", fileObj, swap) ;

    DBC_Require (fileObj != NULL) ;

    KFILE_Read ((Char8 *) ((Pvoid) &retVal),
                READ_REC_SIZE,
                sizeof (Int32),
                fileObj) ;

    if (swap == TRUE) {
        retVal = BYTESWAP_LONG (retVal) ;
    }

    TRC_0LEAVE ("COFF_read32") ;

    return retVal ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
