/** ============================================================================
 *  @file   ldrv_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/ldrv/PrOS/
 *
 *  @desc   Implementation of OS-specific functionality for LDRV.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <print.h>

/*  ----------------------------------- Link Driver                 */
#include <ldrv_os.h>


#if defined (__cplusplus)
extern "C" {
#endif

/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent Identifier.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_LDRV_OS

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_LDRV_OS, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @func   LDRV_getLinkCfg
 *
 *  @desc   Gets the pointer to kernel configuration structure after creating
 *          it (if required).
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
LDRV_getLinkGppCfg (IN  LINKCFG_Object *  linkCfg,
                    OUT LINKCFG_Object ** knlLinkCfg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_2ENTER ("LDRV_getLinkGppCfg", linkCfg, knlLinkCfg) ;

    DBC_Require (linkCfg    != NULL) ;
    DBC_Require (knlLinkCfg != NULL) ;

    /*  ------------------------------------------------------------------------
     *  Directly set the pointer to configuration structure received from API
     *  layer into the LDRV pointer.
     *  ------------------------------------------------------------------------
     */
    *knlLinkCfg = linkCfg ;

    TRC_1LEAVE ("LDRV_getLinkGppCfg", status) ;

    return status ;
}


/** ============================================================================
 *  @func   LDRV_getLinkDspCfg
 *
 *  @desc   Gets the pointer to kernel configuration structure after creating
 *          it (if required).
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
LDRV_getLinkDspCfg (IN  ProcessorId         procId,
                    IN  LINKCFG_DspConfig * dspCfg,
                    OUT LINKCFG_Object *    knlLinkCfg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_3ENTER ("LDRV_getLinkDspCfg",procId, dspCfg, knlLinkCfg) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (dspCfg     != NULL) ;
    DBC_Require (knlLinkCfg != NULL) ;
    
    (Void) dspCfg ;
    (Void) knlLinkCfg ;
    (Void) procId ;

    TRC_1LEAVE ("LDRV_getLinkDspCfg", status) ;

    return status ;
}


/** ============================================================================
 *  @func   LDRV_freeLinkGppCfg
 *
 *  @desc   Frees any memory allocated for the kernel-side DSPLINK configuration
 *          structure.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
LDRV_freeLinkGppCfg (IN  LINKCFG_Object *  knlLinkCfg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("LDRV_freeLinkGppCfg", knlLinkCfg) ;

    /*  ------------------------------------------------------------------------
     *  Nothing to be done for freeing the kernel-side DSPLINK configuration
     *  structure.
     *  ------------------------------------------------------------------------
     */

    TRC_1LEAVE ("LDRV_freeLinkGppCfg", status) ;

    return status ;
}


/** ============================================================================
 *  @func   LDRV_freeLinkCfg
 *
 *  @desc   Frees any memory allocated for the kernel-side DSPLINK configuration
 *          structure.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
LDRV_freeLinkDspCfg (IN  ProcessorId       procId,
                     IN  LINKCFG_Object *  knlLinkCfg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_2ENTER ("LDRV_freeLinkDspCfg", procId, knlLinkCfg) ;

    DBC_Require (IS_VALID_PROCID (procId)) ;
    DBC_Require (knlLinkCfg != NULL) ;
    
    /*  ------------------------------------------------------------------------
     *  Nothing to be done for freeing the kernel-side DSPLINK configuration
     *  structure.
     *  ------------------------------------------------------------------------
     */
    
    (Void) procId ;
    (Void) knlLinkCfg ; 

    TRC_1LEAVE ("LDRV_freeLinkDspCfg", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif
