/** ============================================================================
 *  @file   kfile_prfile.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/PrOS/
 *
 *  @desc   Implementation of sub-component KFILE based on the prfile system.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <itron.h>
#include <pf_api.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Generic Function            */
#include<gen_utils.h>



#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define  COMPONENT_ID  ID_OSAL_KFILE

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason (status, FID_C_OSAL_KFILE, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG)  */


/** ============================================================================
 *  @name   KFILEPRFILE_Object
 *
 *  @field  fileDesc
 *              File descriptor.
 *  @field  size
 *              Size of the file.
 *  @field  curPos
 *              Current file position indicator.
 *
 *  @see    None
 *  ============================================================================
 */
typedef struct KFILEPRFILE_Object_tag {
    PF_FILE *     fp        ;
    Uint32        size      ;
    Uint32        curPos    ;
} KFILEPRFILE_Object ;


/** ============================================================================
 *  @func   KFILEPRFILE_Open
 *
 *  @desc   Opens a file specified by name of file.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Open (IN CONST FileName fileName,
                  IN CONST Char8 *  mode,
                  IN Void **        fileHandlePtr) ;

/** ============================================================================
 *  @func   KFILEPRFILE_Close
 *
 *  @desc   Closes a file handle.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Close (IN Void * fileHandle) ;

/** ============================================================================
 *  @func   KFILEPRFILE_Read
 *
 *  @desc   Reads a specified number of items of specified size
 *          bytes from file to a buffer.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Read (IN OUT  Char8 *       buffer,
                  IN      Uint32        size,
                  IN      Uint32        count,
                  IN      Void *        fileHandle) ;

/** ============================================================================
 *  @func   KFILEPRFILE_Seek
 *
 *  @desc   Repositions the file pointer according to specified arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Seek (IN  Void *          fileHandle,
                  IN  Int32           offset,
                  IN  KFILE_FileSeek  origin) ;


/** ============================================================================
 *  @func   KFILEPRFILE_Tell
 *
 *  @desc   Returns the current file pointer position for the specified
 *          file handle.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Tell (IN  Void *        fileHandle,
                  OUT Int32 *       pos) ;


/** ============================================================================
 *  @func   KFILEPRFILE_GetSize
 *
 *  @desc   Return the size of file in Bytes.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
KFILEPRFILE_GetSize (IN Void * fileHandle,
                     OUT Uint32 * size) ;


/** ============================================================================
 *  @name   KFILEPRFILE_Interface
 *
 *  @desc   Definition of the interface table for KFILEPRFILE module.
 *  ============================================================================
 */
KFILE_Interface KFILEDEF_Interface =
{
    &KFILEPRFILE_Open,
    &KFILEPRFILE_Close,
    &KFILEPRFILE_Read,
    &KFILEPRFILE_Seek,
    &KFILEPRFILE_Tell,
    &KFILEPRFILE_GetSize
} ;


/** ============================================================================
 *  @func   KFILEPRFILE_Open
 *
 *  @desc   Opens a file specified by name of file.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Open (IN CONST FileName fileName,
                  IN CONST Char8 *  mode,
                  IN Void **        fileHandlePtr)
{
    DSP_STATUS  status = DSP_SOK     ;
    Uint32      length = 0           ;
    PF_FILE *   fp     = NULL        ;
    int         erCode = 0           ;
    PF_FPOS_T   currentPosition = 0  ;
    KFILEPRFILE_Object * fileObj = NULL ;

    TRC_3ENTER ("KFILEPRFILE_Open", fileName, mode, fileHandlePtr) ;

    DBC_Require (fileName != NULL) ;
    DBC_Require (mode != NULL) ;
    DBC_Require (fileHandlePtr != NULL) ;

    if (   (fileName == NULL)
        || (fileHandlePtr == NULL)
        || (mode == NULL)
        || (mode [0] != 'r')) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        *fileHandlePtr = NULL ;
        status = GEN_Strlen (fileName, &length) ;
        if (length == 0) {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
        else {
            status = MEM_Alloc ((Void **) &fileObj,
                                sizeof (KFILEPRFILE_Object),
                                MEM_DEFAULT) ;
        }
    }

    if (DSP_SUCCEEDED (status)) {
        *fileHandlePtr = (Void *) fileObj ;
        /*  --------------------------------------------------------------------
         *  Open the file using pf_fopen call. Initialize the file object
         *  after validating the object returned by PrOS kernel.
         *  --------------------------------------------------------------------
         */
        fp = pf_fopen (fileName, "r") ;
        if ( fp == NULL) {
            status = DSP_EFILE ;
            SET_FAILURE_REASON ;
        }
        else {
            (fileObj)->fp        = fp       ;
            (fileObj)->curPos    = 0          ;

            erCode = pf_fseek (fp, 0, PF_SEEK_END) ;
            if (erCode != 0) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else {
                erCode = pf_fgetpos (fp, &currentPosition) ;
                if (erCode != 0) {
                    status = DSP_EFAIL ;
                    SET_FAILURE_REASON ;
                }
                else  {
                    /* Calculating the file size. */
                    (fileObj)->size      = currentPosition ;

                    erCode = pf_fseek (fp, 0, PF_SEEK_SET) ;
                    if (erCode != 0) {
                        status = DSP_EFAIL ;
                        SET_FAILURE_REASON ;
                    }
                }
            }
        }

        /*  --------------------------------------------------------------------
         *  If the function call failed then free the object allocated before.
         *  --------------------------------------------------------------------
         */
        if (DSP_FAILED (status)) {
            FREE_PTR (*fileHandlePtr) ;
        }
    }

    DBC_Ensure (   DSP_SUCCEEDED (status)
                || DSP_FAILED (status)) ;

    DBC_Ensure (   DSP_SUCCEEDED (status)
                || (   DSP_FAILED (status)
                    && (fileHandlePtr != NULL)
                    && (*fileHandlePtr == NULL))) ;

    TRC_1LEAVE ("KFILEPRFILE_Open", status) ;

    return status ;
}


/** ============================================================================
 *  @func   KFILEPRFILE_Close
 *
 *  @desc   Closes a file handle.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Close (IN Void * fileHandle)
{
    DSP_STATUS   status = DSP_SOK ;
    int          erCode = 0       ;
    KFILEPRFILE_Object * fileObj = NULL ;

    TRC_1ENTER ("KFILEPRFILE_Close", fileHandle) ;

    DBC_Require (fileHandle != NULL) ;

    fileObj = (KFILEPRFILE_Object *) fileHandle ;
    DBC_Require (fileObj != NULL) ;

    erCode = pf_fclose (fileObj->fp) ;
    if (erCode == 0) {
        FREE_PTR (fileObj) ;
    }
    else {
        status = DSP_EFAIL ;
        SET_FAILURE_REASON ;
    }

    DBC_Ensure (   (DSP_SUCCEEDED (status) && (fileObj == NULL))
                || (DSP_FAILED (status))) ;

    TRC_1LEAVE ("KFILEPRFILE_Close", status) ;
    return status ;
}


/** ============================================================================
 *  @func   KFILEPRFILE_Read
 *
 *  @desc   Reads a specified number of items of specified size
 *          bytes from file to a buffer.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Read (IN OUT  Char8 *       buffer,
                  IN      Uint32        size,
                  IN      Uint32        count,
                  IN      Void *        fileHandle)
{
    DSP_STATUS      status       = DSP_SOK ;
    PF_SIZE_T       bytesRead    = 0       ;
    Uint32          i            = 0       ;
    PF_SIZE_T       itemCount    = 0 ;
    KFILEPRFILE_Object * fileObj = NULL ;

    TRC_4ENTER ("KFILEPRFILE_Read", buffer, size, count, fileHandle) ;

    DBC_Require (fileHandle != NULL) ;
    DBC_Require (buffer != NULL) ;

    fileObj = (KFILEPRFILE_Object *) fileHandle ;

    if (buffer == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else if ((fileObj->curPos + (size * count)) > fileObj->size) {
        status = DSP_ERANGE ;
        SET_FAILURE_REASON  ;
    }
    else if ((size != 0) && (count != 0)) {
        for (i = 0; i < (count / 128); i++) {
             itemCount = pf_fread ((Pvoid) &buffer [i * 128],
                                   size,
                                   128,
                                   fileObj->fp) ;
            if (itemCount != (PF_SIZE_T)128) {
                status = DSP_EFILE ;
                SET_FAILURE_REASON ;
            }
            else {
                /* Get the bytes read */
                bytesRead += size * ((Uint32)itemCount) ;
            }
        }
        if (count % 128) {
            itemCount = pf_fread ((Pvoid) &buffer [i * 128],
                                  size,
                                  (count % 128),
                                  fileObj->fp) ;
            if (itemCount != (PF_SIZE_T)(count % 128)) {
                status = DSP_EFILE ;
                SET_FAILURE_REASON ;
            }
            else {
                /* Get the bytes read */
                bytesRead += size * ((Uint32)itemCount) ;
            }
        }
        if (bytesRead <= (PF_SIZE_T)0) {
            status = DSP_EFILE ;
            SET_FAILURE_REASON ;
        }
        else {
            /* Update current position */
            fileObj->curPos += bytesRead ;
            DBC_Assert (bytesRead == (Uint32)(count * size)) ;
        }
    }

    return status ;
}


/** ============================================================================
 *  @func   KFILEPRFILE_Seek
 *
 *  @desc   Repositions the file pointer according to specified arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Seek (IN  Void *          fileHandle,
                  IN  Int32           offset,
                  IN  KFILE_FileSeek  origin)
{
    DSP_STATUS    status          = DSP_SOK     ;
    PF_FPOS_T     currentPosition = 0  ;
    int           erCode          = 0           ;
    KFILEPRFILE_Object * fileObj  = NULL ;

    TRC_3ENTER ("KFILEPRFILE_Seek", fileHandle, offset, origin) ;

    DBC_Require (fileHandle != NULL) ;
    DBC_Require (   (origin == KFILE_SeekSet)
                 || (origin == KFILE_SeekCur)
                 || (origin == KFILE_SeekEnd)) ;

    fileObj = (KFILEPRFILE_Object *) fileHandle ;
    DBC_Require (fileObj != NULL) ;

    switch (origin) {
    case KFILE_SeekSet:
        if (   (offset < 0)
            || (((Uint32) (offset)) > fileObj->size)) {
            status = DSP_ERANGE ;
            SET_FAILURE_REASON ;
        }
        else {
            pf_fseek (fileObj->fp, offset, PF_SEEK_SET) ;
            erCode = pf_fgetpos (fileObj->fp, &currentPosition) ;
            if (erCode != 0) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else  {
                fileObj->curPos = currentPosition ;
            }
        }
        break ;

    case KFILE_SeekCur:
        if (   (  (Uint32)(fileObj->curPos + offset))
            > fileObj->size) {
            status = DSP_ERANGE ;
            SET_FAILURE_REASON ;
        }
        else {
            pf_fseek (fileObj->fp, offset, PF_SEEK_CUR) ;
            erCode = pf_fgetpos (fileObj->fp, &currentPosition) ;
            if (erCode != 0) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else  {
                fileObj->curPos = currentPosition ;
            }
        }
        break ;

    case KFILE_SeekEnd:
        /*  ----------------------------------------------------------------
         *  A negative offset indicates offset from the end of file.
         *  Check that the specified offset is not beyond
         *  the bounds of the file.
         *  ----------------------------------------------------------------
         */

        if (   (offset > 0)
            || (((Uint32)(-offset)) > fileObj->size)) {
            status = DSP_ERANGE ;
            SET_FAILURE_REASON ;
        }
        else {
            pf_fseek (fileObj->fp, offset, PF_SEEK_END) ;
            erCode = pf_fgetpos (fileObj->fp, &currentPosition) ;
            if (erCode != 0) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else  {
                fileObj->curPos = currentPosition ;
            }
        }
        break ;

    default:
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
        break ;
    }

    TRC_1LEAVE ("KFILEPRFILE_Seek", status) ;

    return status ;
}


/** ============================================================================
 *  @func   KFILEPRFILE_Tell
 *
 *  @desc   Returns the current file pointer position for the specified
 *          file handle.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
KFILEPRFILE_Tell (IN  Void *        fileHandle,
                  OUT Int32 *       pos)
{
    DSP_STATUS   status          = DSP_SOK    ;
    PF_FPOS_T    currentPosition = 0 ;
    int          erCode          = 0 ;
    KFILEPRFILE_Object * fileObj = NULL ;

    TRC_2ENTER ("KFILEPRFILE_Tell", fileHandle, pos) ;

    DBC_Require (fileHandle != NULL) ;
    DBC_Require (pos != NULL) ;
    fileObj = (KFILEPRFILE_Object *) fileHandle ;

    if (pos == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        *pos = fileObj->curPos ;
        erCode = pf_fgetpos (fileObj->fp, &currentPosition) ;
        if (erCode != 0) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
        else  {
            if (*pos != currentPosition) {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
        }
    }

    DBC_Ensure (   (DSP_SUCCEEDED (status) && (pos != NULL) && (*pos >= 0))
                || (DSP_FAILED (status))) ;

    TRC_1LEAVE ("KFILEPRFILE_Tell", status) ;

    return status ;
}


/** ============================================================================
 *  @func   KFILEPRFILE_GetSize
 *
 *  @desc   Return the size of file in Bytes.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
KFILEPRFILE_GetSize (IN  Void *   fileHandle,
                     OUT Uint32 * size)
{
    KFILEPRFILE_Object * fileObj = NULL ;

    TRC_2ENTER ("KFILEPRFILE_GetSize", fileHandle, size) ;

    DBC_Require (fileHandle != NULL) ;

    fileObj = (KFILEPRFILE_Object *) fileHandle ;

    *size = fileObj->size    ;

    TRC_0LEAVE ("KFILEPRFILE_GetSize") ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
