/** ============================================================================
 *  @file   notify_knl.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/PrOS/
 *
 *  @desc   Implemetation of Kernel-side NOTIFY.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <itron.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>
#include <procdefs.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- Profiling                     */
#if defined (DDSP_PROFILE)
#include <profile.h>
#endif /* #if defined (DDSP_PROFILE) */

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- LINK driver                 */
#include <ips.h>

#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component and Subcomponent ID.
 *  ============================================================================
 */
#define  COMPONENT_ID       ID_OSAL_NOTIFY_KNL

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON  TRC_SetReason \
                                       (status, FID_C_OSAL_NOTIFY_KNL, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @func   NOTIFY_Initialize
 *
 *  @desc   This function initializes the Kernel-side NOTIFY component.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
NOTIFY_KnlInitialize (IN ProcessorId dspId)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("NOTIFY_KnlInitialize", dspId) ;

    /* Nothing to be done for Initialize. */

    TRC_1LEAVE ("NOTIFY_KnlInitialize", status) ;

    return status ;
}

/** ============================================================================
 *  @func   NOTIFY_KnlFinalize
 *
 *  @desc   This function Finalizes the Kernel-side NOTIFY component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
NOTIFY_KnlFinalize (IN ProcessorId dspId)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_1ENTER ("NOTIFY_KnlFinalize", dspId) ;

    /* Nothing to be done for Finalize. */

    TRC_1LEAVE ("NOTIFY_KnlFinalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   NOTIFY_KnlRegister
 *
 *  @desc   This function registers a callback from an event.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
NOTIFY_KnlRegister (IN     ProcessorId   dspId,
                    IN     Uint32        ipsId,
                    IN     Uint32        eventNo,
                    IN     FnNotifyCbck  cbFxn,
                    IN     Pvoid         cbArg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_5ENTER ("NOTIFY_KnlRegister", dspId, ipsId, eventNo, cbFxn, cbArg) ;

    status = IPS_register (dspId, ipsId, eventNo, cbFxn, cbArg) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("NOTIFY_KnlRegister", status) ;

    return status ;
}


/** ============================================================================
 *  @func   NOTIFY_KnlUnregister
 *
 *  @desc   This function unregisters a callback from an event.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
NOTIFY_KnlUnregister (IN     ProcessorId    dspId,
                      IN     Uint32         ipsId,
                      IN     Uint32         eventNo,
                      IN     FnNotifyCbck   cbFxn,
                      IN     Pvoid          cbArg)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_5ENTER ("NOTIFY_KnlUnregister", dspId, ipsId, eventNo, cbFxn, cbArg) ;

    status = IPS_unregister (dspId, ipsId, eventNo, cbFxn, cbArg) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("NOTIFY_KnlUnregister", status) ;

    return status ;
}


/** ============================================================================
 *  @func   NOTIFY_KnlNotify
 *
 *  @desc   This function notifys the other side about an event.
 *
 *  @modif  None.
 *  ============================================================================
 */
NORMAL_API
DSP_STATUS
NOTIFY_KnlNotify (IN     ProcessorId dspId,
                  IN     Uint32      ipsId,
                  IN     Uint32      eventNo,
                  IN     Uint32      payload)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_4ENTER ("NOTIFY_KnlNotify", dspId, ipsId, eventNo, payload) ;

    status = IPS_notify (dspId, ipsId, eventNo, payload, TRUE) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    TRC_1ENTER ("NOTIFY_KnlNotify", status) ;

    return status ;
}


/** ============================================================================
 *  @func   UEVENT_Initialize
 *
 *  @desc   This function initializes the UEVENT notification component.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
Void
UEVENT_Initialize (Void)
{
    TRC_0ENTER ("UEVENT_Initialize") ;

    /* Nothing to be done. */

    TRC_0LEAVE ("UEVENT_Initialize") ;
}


/** ============================================================================
 *  @func   UEVENT_Finalize
 *
 *  @desc   This function finalize the UEVENT notification component.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
Void
UEVENT_Finalize (Void)
{
    TRC_0ENTER ("UEVENT_Finalize") ;

    /* Nothing to be done. */

    TRC_0LEAVE ("UEVENT_Finalize") ;
}


/** ============================================================================
 *  @func   UEVENT_AddNewProcess
 *
 *  @desc   This function adds a new process for getting event nofiicatin.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
UEVENT_AddNewProcess (Void)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_0ENTER ("UEVENT_AddNewProcess") ;

    /* Nothing to be done. */

    TRC_1LEAVE ("UEVENT_AddNewProcess", status) ;

    return status ;
}


/** ============================================================================
 *  @func   UEVENT_RemoveUserProcess
 *
 *  @desc   This function adds a new process for getting event nofiicatin.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
UEVENT_RemoveUserProcess (Void)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_0ENTER ("UEVENT_RemoveUserProcess") ;

    /* Nothing to be done. */

    TRC_1LEAVE ("UEVENT_RemoveUserProcess", status) ;

    return status ;
}


/** ============================================================================
 *  @func   UEVENT_AddBufByPid
 *
 *  @desc   This function adds a data to a registered process.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
UEVENT_AddBufByPid (IN Uint32       pid,
                    IN Uint32       eventNo,
                    IN Uint32       data,
                    IN FnNotifyCbck cbFxn,
                    IN Pvoid        param,
                    IN Uint32       isRingio)
{
    DSP_STATUS status = DSP_SOK ;

    TRC_6ENTER ("UEVENT_AddBufByPid",
                pid,
                eventNo,
                data,
                cbFxn,
                param,
                isRingio) ;

    /* Nothing to be done. */
    (void) pid      ;
    (void) eventNo  ;
    (void) data     ;
    (void) cbFxn    ;
    (void) param    ;
    (void) isRingio ;

    TRC_1LEAVE ("UEVENT_AddBufByPid", status) ;

    return status ;
}


/** ============================================================================
 *  @func   UEVENT_GetBuf
 *
 *  @desc   This function gets a buf if there any.
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
Uint32
UEVENT_GetBuf (OUT Pvoid packet)
{
    Uint32 retVal = 0 ;

    DBC_Require (packet != NULL) ;

    TRC_1ENTER ("UEVENT_GetBuf", packet) ;

    /* Nothing to be done. */

    TRC_1LEAVE ("UEVENT_GetBuf", retVal) ;

    return retVal ;
}


#if defined (DDSP_PROFILE)
/** ============================================================================
 *  @func   NOTIFY_KnlInstrument
 *
 *  @desc   This function gives the profile instrumentation
 *
 *  @modif  None.
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
NOTIFY_KnlInstrument (IN     ProcessorId      dspId,
                      IN     Uint32           ipsId,
                      OUT    IPS_Instrument * retVal)
{
    DSP_STATUS          status = DSP_SOK ;

    DBC_Require (IS_VALID_PROCID (dspId)) ;
    DBC_Require (retVal != NULL) ;

    TRC_3ENTER ("NOTIFY_KnlInstrument", dspId, ipsId, retVal) ;

    status = IPS_instrument (dspId, ipsId, retVal) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("NOTIFY_KnlInstrument", status) ;

    return status ;
}
#endif /* if defined (DDSP_PROFILE) */


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
