/** ============================================================================
 *  @file   sync.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/PrOS/
 *
 *  @desc   Implementation of sub-component SYNC.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <itron.h>
#include <osdefs.h>

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Identifier for sub-component.
 *  ============================================================================
 */
#define COMPONENT_ID        ID_OSAL_SYNC

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_OSAL_SYNC, __LINE__)
#else
#define SET_FAILURE_REASON { }
#endif

/** ============================================================================
 *  @const  EVENT_BIT_PATTERN
 *
 *  @desc   The event flags bit pattern.
 *  ============================================================================
 */
#define EVENT_BIT_PATTERN  0x1u

/** ============================================================================
 *  @const  NO_SUSPEND
 *
 *  @desc   Value to indicate no waiting, when acquiring semaphore.
 *  ============================================================================
 */
#define NO_SUSPEND  0u

/** ============================================================================
 *  @const  SEM_NON_SIGNALLED
 *
 *  @desc   Non signalled semaphore value.
 *  ============================================================================
 */
#define SEM_NON_SIGNALLED    0u

/** ============================================================================
 *  @const  SEM_SIGNALLED
 *
 *  @desc   Signalled semaphore value.
 *  ============================================================================
 */
#define SEM_SIGNALLED    1u

/** ============================================================================
 *  @const  SEM_BINARY
 *
 *  @desc   Maximum count for binary type semaphore.
 *  ============================================================================
 */
#define SEM_BINARY    1u

/** ============================================================================
 *  @const  SEM_COUNTING
 *
 *  @desc   Maximum count for counting type semaphore.
 *  ============================================================================
 */
#define SEM_COUNTING    0xFFFF

/** ============================================================================
 *  @const  IRQ_START_SUCCESS
 *
 *  @desc   Value to indicate the successful operation of SYNC_SpinLockStart.
 *  ============================================================================
 */
#define IRQ_START_SUCCESS    0x1u


/** ============================================================================
 *  @name   SyncEvObject
 *
 *  @desc   This object is used for various event related API.
 *
 *  @field  signature
 *              For identification of this object.
 *  @field  eventId
 *              ID number of event flag.
 *
 *  @see    None
 *  ============================================================================
 */
struct SyncEvObject_tag {
    Uint32            signature       ;
    ID                eventId         ;
} ;

/** ============================================================================
 *  @name   SyncCsObject_tag
 *
 *  @desc   This object is used by various CS API's.
 *
 *  @field  signature
 *              For identification of this object.
 *  @field  mtxId
 *              ID number of mutex.
 *  @field  nestedCount
 *              This variable has been added to support nesting of loc_mtx calls
 *              loc_mtx allows nesting. The second nested call simply returns
 *              E_ILUSE and does not block. The caller should call unl_mtx
 *              only if saved status is E_OK.
 *
 *  @see    None
 *  ============================================================================
 */
struct SyncCsObject_tag {
    Uint32            signature ;
    ID                mtxId     ;
    Uint32            nestedCount     ;
} ;

/** ============================================================================
 *  @name   SyncSemObject
 *
 *  @desc   This object is used by various SEM API's.
 *
 *  @field  signature
 *              For identification of this object.
 *  @field  semType
 *              Indicates the type of the semaphore (binary or counting).
 *  @field  isSemAvailable
 *              Flag to indicate if the binary semaphore is available.
 *              If flag is TRUE then semaphore is available.
 *              If flag is FALSE then semaphore is not available.
 *  @field  semId
 *              ID number of event semaphore.
 *
 *  @see    None
 *  ============================================================================
 */
struct SyncSemObject_tag {
    Uint32            signature         ;
    SyncSemType       semType           ;
    Bool              isSemAvailable    ;
    ID                semId             ;
} ;

/** ----------------------------------------------------------------------------
 *  @name   SYNC_IsInitialized
 *
 *  @desc   Indicates if the module has been initialized or not.
 *  ----------------------------------------------------------------------------
 */
STATIC Bool SYNC_IsInitialized = FALSE ;

/** ----------------------------------------------------------------------------
 *  @name   SYNC_ProtectCsObj
 *
 *  @desc   Handle to the critical section object used for protecting against
 *          other tasks during SYNC_ProtectionStart ()/SYNC_ProtectionEnd ()
 *          calls.
 *  ----------------------------------------------------------------------------
 */
STATIC SyncCsObject * SYNC_ProtectCsObj = NULL ;


/** ============================================================================
 *  @func   SYNC_Initialize
 *
 *  @desc   Initialize SYNC sub-component by allocating all resources.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("SYNC_Initialize") ;

    DBC_Require (SYNC_IsInitialized == FALSE) ;

    if (SYNC_IsInitialized == FALSE) {
        status = SYNC_CreateCS (&SYNC_ProtectCsObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }

        SYNC_IsInitialized = TRUE ;
    }

    TRC_1LEAVE ("SYNC_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_Finalize
 *
 *  @desc   Releases resources used by the sub-component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("SYNC_Finalize") ;

    DBC_Require (SYNC_IsInitialized == TRUE) ;

    if (SYNC_IsInitialized == TRUE) {
        if (SYNC_ProtectCsObj != NULL) {
            status = SYNC_DeleteCS (SYNC_ProtectCsObj) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
            SYNC_ProtectCsObj = NULL ;
        }

        SYNC_IsInitialized = FALSE ;
    }

    TRC_1LEAVE ("SYNC_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_OpenEvent
 *
 *  @desc   Creates and initializes an event object for thread
 *          synchronization. The event is initialized to a
 *          'non-signaled' state.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_OpenEvent (OUT SyncEvObject ** event,
                IN  SyncAttrs *     attr)
{
    DSP_STATUS   status   = DSP_SOK ;
    ER_ID        erId     = E_OK    ;
    T_CFLG       pk_cflag           ;

    TRC_2ENTER ("SYNC_OpenEvent", event, attr) ;

    DBC_Require (event != NULL) ;
    DBC_Require (attr != NULL) ;

    if (attr == NULL) {
        if (event != NULL) {
            *event = NULL ;
        }
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else if (event == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status = MEM_Calloc ((Void **) event, sizeof (SyncEvObject),
                             MEM_DEFAULT) ;
        if (DSP_SUCCEEDED (status)) {
            /** ----------------------------------------------------------------
             *  Populate the event flag creation information structure.
             *  ----------------------------------------------------------------
             */
            pk_cflag.flgatr  = (TA_TPRI | TA_WMUL | TA_CLR) ;
            pk_cflag.iflgptn = 0    ;

            /** ----------------------------------------------------------------
             *  Create the event flag.
             *  ----------------------------------------------------------------
             */
            erId = acre_flg (&pk_cflag) ;

            if (erId < 0) {
                FREE_PTR (*event) ;
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
            else {
                (*event)->signature = SIGN_SYNC ;
                /* erId contains a valid event flag id. */
                (*event)->eventId   = erId      ;
            }
        }
        else {
            SET_FAILURE_REASON ;
        }
    }

    /*  DBC to check consistency in case of failure */
    DBC_Ensure (   (DSP_SUCCEEDED (status))
                || (   (event == NULL)
                    || (!IS_OBJECT_VALID ((*event), SIGN_SYNC)))) ;

    /*  DBC to check consistency in case of success */
    DBC_Ensure (   (DSP_FAILED (status))
                || (   (event != NULL)
                    && IS_OBJECT_VALID ((*event), SIGN_SYNC))) ;


    TRC_1LEAVE ("SYNC_OpenEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_CloseEvent
 *
 *  @desc   Closes the handle corresponding to an event. It also frees the
 *          resources allocated, if any, during call to SYNC_OpenEvent ().
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_CloseEvent (IN SyncEvObject * event)
{
    DSP_STATUS status    = DSP_SOK ;
    DSP_STATUS tmpStatus = DSP_SOK ;
    ER         erStatus  = E_OK    ;

    TRC_1ENTER ("SYNC_CloseEvent", event) ;

    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;
    DBC_Require (event != NULL) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        event->signature = SIGN_NULL ;
    }
    else {
        SET_FAILURE_REASON ;
    }

    if (DSP_SUCCEEDED (status)) {
        /** --------------------------------------------------------------------
         *  Delete the event flag.
         *  --------------------------------------------------------------------
         */
        erStatus = del_flg (event->eventId) ;
        if (erStatus != E_OK) {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }

        tmpStatus = FREE_PTR (event) ;
        if ((DSP_SUCCEEDED (status)) && (DSP_FAILED (tmpStatus))) {
            status = tmpStatus ;
            SET_FAILURE_REASON ;
        }
    }

    TRC_1LEAVE ("SYNC_CloseEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_ResetEvent
 *
 *  @desc   Reset the synchronization object to 'non-signaled' state.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_ResetEvent (IN SyncEvObject * event)
{
    DSP_STATUS status   = DSP_SOK ;
    ER         erStatus = E_OK    ;

    TRC_1ENTER ("SYNC_ResetEvent", event) ;

    DBC_Require (event != NULL) ;
    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        /** --------------------------------------------------------------------
         *  Reset the event flag bit pattern.
         *  --------------------------------------------------------------------
         */
        erStatus = clr_flg (event->eventId, ~EVENT_BIT_PATTERN) ;
        if (erStatus != E_OK) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_ResetEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_SetEvent
 *
 *  @desc   Set the state of synchronization object to 'signaled' and unblock
 *          all threads waiting for it.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_SetEvent (IN SyncEvObject * event)
{
    DSP_STATUS status   = DSP_SOK ;
    ER         erStatus = E_OK    ;

    TRC_1ENTER ("SYNC_SetEvent", event) ;

    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;
    DBC_Require (event != NULL) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        erStatus = set_flg (event->eventId, EVENT_BIT_PATTERN) ;
        if (erStatus != E_OK) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_SetEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_WaitOnEvent
 *
 *  @desc   Wait for an event to be 'signaled' for a specified amount of time.
 *          It is possible to wait infinitely. This function must 'block'
 *          and not 'spin'.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_WaitOnEvent (IN SyncEvObject * event,
                  IN Uint32         timeOut)
{
    DSP_STATUS  status      = DSP_SOK   ;
    ER          erStatus    = E_OK      ;
    Uint32      suspendTime             ;
    FLGPTN      flgptn                  ;

    TRC_2ENTER ("SYNC_WaitOnEvent", event, timeOut) ;

    DBC_Require (event != NULL) ;
    DBC_Require (IS_OBJECT_VALID (event, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (event, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    /*  ------------------------------------------------------------------------
     *  Map 'timeOut' to NO_SUSPEND (= 0) or WAIT_FOREVER (= 0xFFFFFFFFUL)
     *  ------------------------------------------------------------------------
     */
    if (DSP_SUCCEEDED (status)) {
        if (timeOut == SYNC_WAITFOREVER) {
            suspendTime = WAIT_FOREVER ;
            erStatus = wai_flg (event->eventId,
                                EVENT_BIT_PATTERN,
                                TWF_ANDW,
                                &flgptn) ;
        }
        else {
            if (timeOut == SYNC_NOWAIT) {
                suspendTime = NO_SUSPEND ;
            }
            else {
                suspendTime = (timeOut * TICKS_PER_MILLI_SEC) ;
            }
            erStatus = twai_flg (event->eventId,
                                 EVENT_BIT_PATTERN,
                                 TWF_ANDW,
                                 &flgptn,
                                 suspendTime) ;
        }


        if (erStatus != E_OK) {
            /*  ----------------------------------------------------------------
             *  Out of all the *NOT* E_OK errors, set the status only if
             *  the erStatus is E_TMOUT.
             *  ----------------------------------------------------------------
             */
            if ((erStatus == E_TMOUT) && (timeOut == NO_SUSPEND)) {
                status = SYNC_E_FAIL ;
                SET_FAILURE_REASON ;
            }
            else if (erStatus == E_TMOUT) {
                status = DSP_ETIMEOUT ;
                SET_FAILURE_REASON ;
            }
            else if ((erStatus == E_NOEXS) || (erStatus == E_ID)) {
                status = SYNC_E_FAIL ;
                SET_FAILURE_REASON ;
            }
            else {
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
            }
        }

        if (erStatus != E_OK) {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_WaitOnEvent", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_WaitOnMultipleEvents
 *
 *  @desc   Waits on muliple events. Returns when any of the event is set.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_WaitOnMultipleEvents (IN  SyncEvObject **   syncEvents,
                           IN  Uint32            count,
                           IN  Uint32            timeOut,
                           OUT Uint32 *          index)
{
    DSP_STATUS      status = DSP_SOK ;

    TRC_4ENTER ("SYNC_WaitOnMultipleEvents", syncEvents, count,
                                             timeOut, index) ;
    TRC_1LEAVE ("SYNC_WaitOnMultipleEvents", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_CreateCS
 *
 *  @desc   Creates the Critical section structure.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_CreateCS (OUT SyncCsObject ** csObj)
{
    DSP_STATUS   status   = DSP_SOK ;
    ER_ID        erId     = E_OK    ;
    T_CMTX       pk_cmtx            ;

    TRC_1ENTER ("SYNC_CreateCS", csObj) ;

    DBC_Require (csObj != NULL) ;

    if (csObj == NULL) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else {
        status =  MEM_Calloc ((Void **) csObj, sizeof (SyncCsObject),
                              MEM_DEFAULT) ;
    }

    if (DSP_SUCCEEDED (status)) {
        pk_cmtx.mtxatr  = TA_TFIFO     ;
        pk_cmtx.ceilpri = 0            ;

        /** ----------------------------------------------------------------
         *  Create the event semaphore.
         *  ----------------------------------------------------------------
         */
        erId = acre_mtx (&pk_cmtx) ;

        if (erId <= 0) {
            FREE_PTR (*csObj) ;
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }
        else {
            /* erId contains a valid semaphore id. */
            (*csObj)->mtxId     = erId      ;
            (*csObj)->signature = SIGN_SYNC ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    DBC_Ensure (   ((csObj == NULL) && DSP_FAILED (status))
                 || (((*csObj)->signature == SIGN_NULL) && DSP_FAILED (status))
                 || (   ((*csObj)->signature == SIGN_SYNC)
                     && DSP_SUCCEEDED (status))) ;

    TRC_1LEAVE ("SYNC_CreateCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_DeleteCS
 *
 *  @desc   Deletes the critical section object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_DeleteCS (IN SyncCsObject * csObj)
{
    DSP_STATUS   status    = DSP_SOK    ;
    DSP_STATUS   tmpStatus = DSP_SOK    ;
    ER           erStatus  = E_OK       ;

    TRC_1ENTER ("SYNC_DeleteCS", csObj) ;

    DBC_Require (csObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (csObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (csObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        /** --------------------------------------------------------------------
         *  Delete the mutex.
         *  --------------------------------------------------------------------
         */
        erStatus = del_mtx (csObj->mtxId) ;
        if (erStatus != E_OK) {
            status = DSP_EINVALIDARG ;
            SET_FAILURE_REASON ;
        }

        csObj->signature = SIGN_NULL ;
        tmpStatus = FREE_PTR (csObj) ;
        if ((DSP_SUCCEEDED (status)) && (DSP_FAILED (tmpStatus))) {
            status = tmpStatus ;
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_DeleteCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_EnterCS
 *
 *  @desc   This function enters the critical section that is passed as
 *          argument to it. This is a blocking call. After successful return of
 *          this function no other thread can enter until this thread exit CS by
 *          calling SYNC_LeaveCS.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_EnterCS (IN SyncCsObject * csObj)
{
    DSP_STATUS   status   = DSP_SOK    ;
    ER           erStatus = E_OK       ;

    TRC_1ENTER ("SYNC_EnterCS", csObj) ;

    DBC_Require (csObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (csObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (csObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        /*  --------------------------------------------------------------------
         *  This is a blocking call . Acquire the semaphore.
         *  --------------------------------------------------------------------
         */
        erStatus = loc_mtx (csObj->mtxId) ;
        status = (((erStatus == E_OK) || (erStatus == E_ILUSE)) ? DSP_SOK : SYNC_E_FAIL) ;
        if (erStatus == E_ILUSE)
        {
            csObj->nestedCount ++ ;
        }

        /*  Call to loc_mtx failed */
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_EnterCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_LeaveCS
 *
 *  @desc   This function makes critical section available for other threads
 *          to enter.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_LeaveCS (IN SyncCsObject * csObj)
{
    DSP_STATUS   status   = DSP_SOK    ;
    ER           erStatus = E_OK       ;

    TRC_1ENTER ("SYNC_LeaveCS", csObj) ;

    DBC_Require (csObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (csObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (csObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        /*  --------------------------------------------------------------------
         *  release the semaphore.
         *  --------------------------------------------------------------------
         */
        if (csObj->nestedCount == 0)
        {
            erStatus = unl_mtx (csObj->mtxId) ;
            status = ((erStatus == E_OK) ? DSP_SOK : SYNC_E_FAIL) ;
            if (DSP_FAILED (status)) {
                SET_FAILURE_REASON ;
            }
        }
        else {
            csObj->nestedCount -- ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_LeaveCS", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_CreateSEM
 *
 *  @desc   Creates the semaphore object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_CreateSEM (OUT SyncSemObject ** semObj, IN SyncAttrs * attr)
{
    DSP_STATUS   status   = DSP_SOK    ;
    ER_ID        erId     = E_OK       ;
    T_CSEM       pk_csem               ;

    TRC_2ENTER ("SYNC_CreateSEM", semObj, attr) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (attr != NULL) ;

    if ((semObj == NULL) || (attr == NULL)) {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }
    else if (   ((SyncSemType)(attr->flag) != SyncSemType_Binary)
             && ((SyncSemType)(attr->flag) != SyncSemType_Counting))  {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        status =  MEM_Calloc ((Void **) semObj, sizeof (SyncSemObject),
                              MEM_DEFAULT) ;

        if (DSP_SUCCEEDED (status)) {
            (*semObj)->semType = (SyncSemType)(attr->flag) ;

            if ((*semObj)->semType == SyncSemType_Binary) {
                pk_csem.sematr  = TA_TFIFO ;
                pk_csem.isemcnt = SEM_NON_SIGNALLED ;
                pk_csem.maxsem  = SEM_BINARY   ;
                (*semObj)->isSemAvailable = FALSE ;
            }
            else {
                pk_csem.sematr  = TA_TFIFO ;
                pk_csem.isemcnt = (UINT) SEM_NON_SIGNALLED   ;
                pk_csem.maxsem  = (UINT) SEM_COUNTING ;
                (*semObj)->isSemAvailable = TRUE ;
            }
            /** ----------------------------------------------------------------
             *  Create the semaphore.
             *  ----------------------------------------------------------------
             */
            erId = acre_sem (&pk_csem) ;

            if (erId <= 0) {
                FREE_PTR (*semObj) ;
                status = DSP_EINVALIDARG ;
                SET_FAILURE_REASON ;
            }
            else {
                /* erId contains a valid semaphore id. */
                (*semObj)->semId     = erId      ;
            }
        }

        if (DSP_SUCCEEDED (status)) {
            (*semObj)->signature = SIGN_SYNC ;
        }
    }

    if ((DSP_FAILED (status)) && (semObj != NULL)) {
        *semObj = NULL ;
    }

    DBC_Ensure (   ((semObj == NULL) && DSP_FAILED (status))
                 || ((*semObj == NULL) && DSP_FAILED (status))
                 || (   ((*semObj)->signature == SIGN_SYNC)
                     && DSP_SUCCEEDED (status))) ;

    TRC_1LEAVE ("SYNC_CreateSEM", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_DeleteSEM
 *
 *  @desc   Deletes the semaphore object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_DeleteSEM (IN SyncSemObject * semObj)
{
    DSP_STATUS   status   = DSP_SOK    ;
    ER           erStatus = E_OK       ;

    TRC_1ENTER ("SYNC_DeleteSEM", semObj) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (semObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (semObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        erStatus = del_sem (semObj->semId) ;

        if (erStatus == E_OK) {
            semObj->signature = SIGN_NULL ;
            FREE_PTR (semObj) ;
        }
        else {
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_DeleteSEM", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_WaitSEM
 *
 *  @desc   This function waits on the semaphore.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_WaitSEM (IN SyncSemObject * semObj, IN Uint32  timeout)
{
    DSP_STATUS   status         = DSP_SOK    ;
    ER           erStatus       = E_OK       ;
    Bool         semAvailable   = FALSE      ;
    Uint32       realTimeout    = 0          ;
    Uint32       irqFlags                    ;

    TRC_2ENTER ("SYNC_WaitSEM", semObj, timeout) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (semObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (semObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        if (semObj->semType == SyncSemType_Binary) {
            irqFlags = SYNC_SpinLockStart () ;
            if (semObj->isSemAvailable == TRUE) {
                semObj->isSemAvailable = FALSE ;
                semAvailable = TRUE ;
                pol_sem (semObj->semId) ;
            }
            SYNC_SpinLockEnd (irqFlags) ;
        }

        while ((DSP_SUCCEEDED (status)) && (semAvailable != TRUE)) {
            if (timeout == SYNC_WAITFOREVER) {
                erStatus = wai_sem (semObj->semId) ;
            }
            else {
                if (timeout == SYNC_NOWAIT) {
                    realTimeout = NO_SUSPEND ;
                }
                else {
                    realTimeout = timeout * TICKS_PER_MILLI_SEC ;
                }

                erStatus = twai_sem (semObj->semId, realTimeout) ;
            }

          if (erStatus == E_OK) {
                if (semObj->semType == SyncSemType_Binary) {
                    irqFlags = SYNC_SpinLockStart () ;
                    if (semObj->isSemAvailable == TRUE) {
                        semObj->isSemAvailable = FALSE ;
                        semAvailable = TRUE ;
                    }
                    SYNC_SpinLockEnd (irqFlags) ;
                }
                else {
                    semAvailable = TRUE ;
                }
            }
            else {
                if (timeout == SYNC_NOWAIT) {
                    status = SYNC_E_FAIL ;
                }
                else {
                    status = DSP_ETIMEOUT ;
                }
                SET_FAILURE_REASON ;
            }
        }
    }

    TRC_1LEAVE ("SYNC_WaitSEM", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_SignalSEM
 *
 *  @desc   This function signals the semaphore and makes it available for other
 *          threads.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_SignalSEM (IN SyncSemObject * semObj)
{
    DSP_STATUS   status   = DSP_SOK    ;
    ER           erStatus = E_OK       ;
    Uint32       irqFlags              ;
    BOOL         state              ;

    TRC_1ENTER ("SYNC_SignalSEM", semObj) ;

    DBC_Require (semObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (semObj, SIGN_SYNC)) ;

    status = (IS_OBJECT_VALID (semObj, SIGN_SYNC) ? DSP_SOK : DSP_EPOINTER) ;

    if (DSP_SUCCEEDED (status)) {
        if (semObj->semType == SyncSemType_Counting) {
            /* get the current context. */
            state = sns_ctx () ;

            if (state == FALSE) {
                erStatus = sig_sem (semObj->semId) ;
            }
            else {
                erStatus = isig_sem (semObj->semId) ;
            }

            if (erStatus != E_OK) {
                status = erStatus ;
                SET_FAILURE_REASON ;
            }
        }
        else {
            irqFlags = SYNC_SpinLockStart () ;
            if (semObj->isSemAvailable != TRUE) {
                semObj->isSemAvailable = TRUE ;
                /* Now make binary semaphore 1 */
                /* get the current context. */
                state = sns_ctx () ;

                if (state == FALSE) {
                    erStatus = sig_sem (semObj->semId) ;
                }
                else {
                    erStatus = isig_sem (semObj->semId) ;
                }

                if (erStatus != E_OK) {
                    status = erStatus ;
                    SET_FAILURE_REASON ;
                }
            }
            SYNC_SpinLockEnd (irqFlags) ;
        }
    }
    else {
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_SignalSEM", status) ;

    return status ;
}


/** ============================================================================
 *  @func   SYNC_ProtectionStart
 *
 *  @desc   Marks the start of protected code execution.
 *          This function achieves protection by disabling DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_ProtectionStart ()
{
    DSP_STATUS status = DSP_SOK ;
    TRC_0ENTER ("SYNC_ProtectionStart") ;

    status = SYNC_EnterCS (SYNC_ProtectCsObj) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    DPC_Disable () ;

    TRC_0LEAVE ("SYNC_ProtectionStart") ;
}


/** ============================================================================
 *  @func   SYNC_ProtectionEnd
 *
 *  @desc   Marks the end of protected code execution.
 *          This function enables DPCs.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_ProtectionEnd ()
{
    DSP_STATUS status = DSP_SOK ;
    TRC_0ENTER ("SYNC_ProtectionEnd") ;

    DPC_Enable () ;

    status = SYNC_LeaveCS (SYNC_ProtectCsObj) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }

    TRC_0LEAVE ("SYNC_ProtectionEnd") ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockCreate
 *
 *  @desc   Creates the Spinlock Object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Pvoid
SYNC_SpinLockCreate (Void)
{
    DSP_STATUS            status = DSP_SOK ;
    Uint32 *              spinLockHandle = NULL ;

    TRC_0ENTER ("SYNC_SpinLockCreate") ;

    status = MEM_Alloc ((Pvoid *) &spinLockHandle,
                        sizeof (Uint32),
                        MEM_DEFAULT) ;
    if (DSP_FAILED (status)) {
        SET_FAILURE_REASON ;
    }
    else {
        /* initialize the object */
        *spinLockHandle = 0 ;
    }

    TRC_1LEAVE ("SYNC_SpinLockCreate", spinLockHandle) ;

    return ((Pvoid) spinLockHandle) ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockDelete
 *
 *  @desc   Deletes the Spinlock Object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_SpinLockDelete (IN Pvoid spinLockHandle)
{
    TRC_1ENTER ("SYNC_SpinLockDelete", spinLockHandle) ;

    if (spinLockHandle != NULL) {
        FREE_PTR (spinLockHandle) ;
    }

    TRC_0LEAVE ("SYNC_SpinLockDelete") ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockStart
 *
 *  @desc   Begin protection of code through loc_cpu with
 *          all ISRs disabled. Calling this API protects critical regions of
 *          code from preemption by tasks, DPCs and all interrupts.
 *          This API can be called from DPC context.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Uint32
SYNC_SpinLockStartEx (IN     Pvoid spinLockHandle)
{
    ER          erStatus = E_OK    ;
    Uint32      irqFlags = 0       ;
    BOOL        state              ;
#if defined (DDSP_DEBUG)
    DSP_STATUS  status   = DSP_SOK ;
#endif /* if defined (DDSP_DEBUG) */

    TRC_1ENTER ("SYNC_SpinLockStartEx", spinLockHandle) ;

    /* get the current context. */
    state = sns_ctx () ;

    if (state == FALSE) {
        /* It is a task context. */
        erStatus = loc_cpu () ;
        if (erStatus != E_OK) {
#if defined (DDSP_DEBUG)
            status = DSP_EFAIL ;
            SET_FAILURE_REASON ;
#endif /* if defined (DDSP_DEBUG) */
        }
        else {
            /* Set irqFlags to indicate that spinlockstart is successful. */
            irqFlags = IRQ_START_SUCCESS ;
        }
    }

    if (spinLockHandle != NULL) {
        *((Uint32 *)spinLockHandle) = irqFlags ;
    }

    TRC_1LEAVE ("SYNC_SpinLockStart", irqFlags) ;

    return irqFlags ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockEnd
 *
 *  @desc   End protection of code through unl_cpu with
 *          all ISRs enabled.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_SpinLockEndEx (IN     Pvoid spinLockHandle, IN     Uint32 irqFlags)
{
    ER          erStatus = E_OK    ;
    BOOL        state              ;
    Uint32      tmpIrqFlags        ;
#if defined (DDSP_DEBUG)
    DSP_STATUS  status   = DSP_SOK ;
#endif /* if defined (DDSP_DEBUG) */

    TRC_1ENTER ("SYNC_SpinLockEnd", irqFlags) ;

    if (spinLockHandle != NULL) {
        tmpIrqFlags = *((Uint32 *)spinLockHandle) ;
    }
    else {
        tmpIrqFlags = irqFlags ;
    }

    if (tmpIrqFlags == IRQ_START_SUCCESS) {
        /* get the current context. */
        state = sns_ctx () ;

        if (state == FALSE) {
            /* It is a task context. */
            erStatus = unl_cpu () ;
            if (erStatus != E_OK) {
#if defined (DDSP_DEBUG)
                status = DSP_EFAIL ;
                SET_FAILURE_REASON ;
#endif /* if defined (DDSP_DEBUG) */
            }
        }
    }

    if (spinLockHandle != NULL) {
        *((Uint32 *)spinLockHandle) = ~tmpIrqFlags ;
    }

    TRC_0LEAVE ("SYNC_SpinLockEnd") ;
}


/** ============================================================================
 *  @func   SYNC_SpinLockEnd
 *
 *  @desc   Pause for number microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
SYNC_UDelay (IN Uint32 numUSec)
{
    TRC_1ENTER ("SYNC_USec", numUSec) ;

    (Void) numUSec ;

    TRC_0LEAVE ("SYNC_UDelay") ;
}


/** ============================================================================
 *  @func   SYNC_SetPriority
 *
 *  @desc   Change the priority of the current  task.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS SYNC_SetPriority (IN Uint32 priority)
{
    ER           erStatus = E_OK ;
    DSP_STATUS   status   = DSP_SOK ;

    TRC_1ENTER ("SYNC_SetPriority", priority) ;

    erStatus = chg_pri (TSK_SELF, priority);
    if(erStatus != E_OK)
    {
        status=DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("SYNC_SetPriority", status) ;

    return status;
}

/** ============================================================================
 *  @func   SYNC_GetPriority
 *
 *  @desc   Get the priority of the current task.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
SYNC_GetPriority (OUT  Uint32 * priority)
{
    DSP_STATUS   status   = DSP_SOK ;
    ER           erStatus = E_OK ;
    PRI          p_tskpri ;

    TRC_0ENTER ("SYNC_GetPriority") ;


    erStatus = get_pri (TSK_SELF, &p_tskpri) ;
    if (erStatus != E_OK) {
        status = DSP_EFAIL ;
        SET_FAILURE_REASON ;
    }
    else {
        *priority = (Uint32) p_tskpri;
    }

    TRC_1LEAVE ("SYNC_GetPriority", status) ;
    return status;
}

#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
