/** ============================================================================
 *  @file   prcs.c
 *
 *  @path   $(DSPLINK)/gpp/src/osal/WinCE/
 *
 *  @desc   Implementation of sub-component PRCS.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers         */
#include <windows.h>
#include <pkfuncs.h> 

/*  ----------------------------------- DSP/BIOS Link               */
#include <dsplink.h>
#include <_dsplink.h>

/*  ----------------------------------- Trace & Debug               */
#include <_trace.h>
#include <trc.h>
#include <print.h>

/*  ----------------------------------- OSAL Headers                */
#include <osal.h>

/*  ----------------------------------- Generic Functions           */



#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */


/** ============================================================================
 *  @macro  COMPONENT_ID
 *
 *  @desc   Component ID of this module.
 *  ============================================================================
 */
#define  COMPONENT_ID        ID_OSAL_PRCS

/** ============================================================================
 *  @macro  SET_FAILURE_REASON
 *
 *  @desc   Sets failure reason.
 *  ============================================================================
 */
#if defined (DDSP_DEBUG)
#define SET_FAILURE_REASON   TRC_SetReason (status, FID_C_OSAL_PRCS, __LINE__)
#else
#define SET_FAILURE_REASON
#endif /* if defined (DDSP_DEBUG) */


/** ============================================================================
 *  @name   PrcsObject
 *
 *  @desc   Structure to store information regarding current process/thread.
 *          This structure is specific to WinCE.
 *
 *  @field  signature
 *              Signature of this structure.
 *  @field  handleToProcess
 *              Handle to current process.
 *  @field  handleToThread
 *              Handle to current thread.
 *  @field  priorityOfProcess
 *              Priority of current process.
 *  @field  priorityOfThread
 *              Priority of current Thread.
 *  ============================================================================
 */
struct PrcsObject_tag {
    Uint32 signature         ;
    Void * handleToProcess   ;
    Void * handleToThread    ;
    Int32  priorityOfProcess ;
    Int32  priorityOfThread  ;

    Void * handleToDLL       ;
} ;


/** ============================================================================
 *  @name   PRCS_IsInitialized
 *
 *  @desc   Flag to keep track of initialization of this subcomponent.
 *  ============================================================================
 */
STATIC Bool PRCS_IsInitialized = FALSE ;


/** ============================================================================
 *  @func   PRCS_Initialize
 *
 *  @desc   Initializes the PRCS subcomponent.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Initialize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("PRCS_Initialize") ;

    DBC_Require (PRCS_IsInitialized == FALSE) ;

    PRCS_IsInitialized = TRUE ;

    TRC_1LEAVE ("PRCS_Initialize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_Finalize
 *
 *  @desc   Finalizes the PRCS component.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Finalize ()
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_0ENTER ("PRCS_Finalize") ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;

    PRCS_IsInitialized = FALSE ;

    TRC_1LEAVE ("PRCS_Finalize", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_Create
 *
 *  @desc   Creates a PrcsObject and populates it with information to identify
 *          the client.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Create (OUT PrcsObject ** prcsObj, IN     Void * optArgs)
{
    DSP_STATUS     status                  = DSP_SOK    ;

    TRC_1ENTER ("PRCS_Create", prcsObj) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (prcsObj != NULL) ;



    if (prcsObj == NULL) {
        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        *prcsObj = NULL ;

        status = MEM_Alloc ((Void **) prcsObj,
                            sizeof (PrcsObject),
                            MEM_DEFAULT) ;

        if (DSP_SUCCEEDED (status)) {
            (*prcsObj)->signature         = SIGN_PRCS;
            (*prcsObj)->handleToThread    = (Pvoid) GetCurrentThreadId();
            (*prcsObj)->handleToProcess   = (Pvoid) GetCallerVMProcessId();
            (*prcsObj)->priorityOfThread  = 0;
            (*prcsObj)->priorityOfProcess = GetThreadPriority(GetCurrentThread());

            (*prcsObj)->handleToDLL       = optArgs;
        }
        else {
            SET_FAILURE_REASON ;
        }
    }



    DBC_Ensure (   (   DSP_SUCCEEDED (status)
                    && (prcsObj != NULL)
                    && IS_OBJECT_VALID(*prcsObj, SIGN_PRCS))
                || DSP_FAILED (status)) ;

    TRC_1LEAVE ("PRCS_Create", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_Delete
 *
 *  @desc   Frees up resources used by the specified object.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_Delete (IN PrcsObject * prcsObj)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_1ENTER ("PRCS_Delete", prcsObj) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (prcsObj != NULL) ;
    DBC_Require (IS_OBJECT_VALID (prcsObj, SIGN_PRCS)) ;

    if (IS_OBJECT_VALID (prcsObj, SIGN_PRCS)) {
        prcsObj->signature = SIGN_NULL ;
        status = FREE_PTR (prcsObj) ;
        if (DSP_FAILED (status)) {
            SET_FAILURE_REASON ;
        }
    }
    else {
        status = DSP_EPOINTER ;
        SET_FAILURE_REASON ;
    }

    TRC_1LEAVE ("PRCS_Delete", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_IsEqual
 *
 *  @desc   Compares two clients to check if they are "equal".
 *          Equality is defined by equality between the process id and
 *          thread id.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_IsEqual (IN  PrcsObject *  client1,
              IN  PrcsObject *  client2,
              OUT Bool *        isEqual)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_3ENTER ("PRCS_IsEqual", client1, client2, isEqual) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (IS_OBJECT_VALID (client1, SIGN_PRCS)) ;
    DBC_Require (IS_OBJECT_VALID (client2, SIGN_PRCS)) ;
    DBC_Require (isEqual != NULL) ;

    if (   (IS_OBJECT_VALID (client1, SIGN_PRCS) == FALSE)
        || (IS_OBJECT_VALID (client2, SIGN_PRCS) == FALSE)
        || (isEqual == NULL)) {

        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {
        if (   (client1->handleToProcess == client2->handleToProcess)

            && (client1->handleToThread  == client2->handleToThread)
            && (client1->handleToDLL     == client2->handleToDLL)) {
            *isEqual = TRUE ;
        }
        else {
            *isEqual = FALSE ;
        }
    }

    TRC_1LEAVE ("PRCS_IsEqual", status) ;

    return status ;
}


/** ============================================================================
 *  @func   PRCS_IsSameContext
 *
 *  @desc   Checks if the two clients share same context (address space).
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
DSP_STATUS
PRCS_IsSameContext (IN  PrcsObject *  client1,
                    IN  PrcsObject *  client2,
                    OUT Bool *        isSame)
{
    DSP_STATUS  status = DSP_SOK ;

    TRC_3ENTER ("PRCS_IsSameContext", client1, client2, isSame) ;

    DBC_Require (PRCS_IsInitialized == TRUE) ;
    DBC_Require (IS_OBJECT_VALID (client1, SIGN_PRCS)) ;
    DBC_Require (IS_OBJECT_VALID (client2, SIGN_PRCS)) ;
    DBC_Require (isSame != NULL) ;

    if (   (IS_OBJECT_VALID (client1, SIGN_PRCS) == FALSE)
        || (IS_OBJECT_VALID (client2, SIGN_PRCS) == FALSE)
        || (isSame == NULL)) {

        status = DSP_EINVALIDARG ;
        SET_FAILURE_REASON ;
    }
    else {

        if (   (client1->handleToProcess == client2->handleToProcess)
            && (client1->handleToDLL     == client2->handleToDLL)){
            *isSame = TRUE ;
        }
        else {
            *isSame = FALSE ;
        }
    }

    TRC_1LEAVE ("PRCS_IsSameContext", status) ;

    return status ;
}


#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
