/** ============================================================================
 *  @file   message_os.c
 *
 *  @path   $(DSPLINK)/gpp/src/samples/message_multidsp/WinCE/
 *
 *  @desc   OS specific implementation of functions used by the message
 *          sample application.
 *
 *  @ver    1.65.02.09
 *  ============================================================================
 *  Copyright (C) 2002-2012, Texas Instruments Incorporated -
 *  http://www.ti.com/
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  
 *  *  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  
 *  *  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  
 *  *  Neither the name of Texas Instruments Incorporated nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  ============================================================================
 */


/*  ----------------------------------- OS Specific Headers           */
#include <windows.h>

/*  ----------------------------------- DSP/BIOS Link                 */
#include <dsplink.h>

/*  ----------------------------------- Application Header            */
#include <message.h>
#include <message_os.h>


#if defined (__cplusplus)
extern "C" {
#endif /* defined (__cplusplus) */

#if defined (MESSAGE_PROFILE)
/** ============================================================================
 *  @name   startUsecTime
 *
 *  @desc   This variable stores the value of start time in microseconds.
 *  ============================================================================
 */
STATIC Uint32 startMsecTime = 0 ;

/** ============================================================================
 *  @name   endUsecTime
 *
 *  @desc   This variable stores the value of end time in microseconds.
 *  ============================================================================
 */
STATIC Uint32 endMsecTime = 0 ;

#endif /* if defined (MESSAGE_PROFILE) */

/** ============================================================================
 *  @name   MAXSTRLENGTH
 *
 *  @desc   Maximum Length of the Trace String.
 *  ============================================================================
 */
#define MAXSTRLENGTH        256


/** ============================================================================
 *  @func   MESSAGEMULTIDSP_0Print
 *
 *  @desc   Print a message without any arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_0Print (Char8 * str)
{
    TCHAR  buffer[MAXSTRLENGTH];    
    mbstowcs (buffer, (const char *)str, MAXSTRLENGTH);
    RETAILMSG(1, (TEXT("%s"), buffer));
}


/** ============================================================================
 *  @func   MESSAGEMULTIDSP_1Print
 *
 *  @desc   Print a message with one arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_1Print (Char8 * str, Uint32 arg)
{
    TCHAR  buffer[MAXSTRLENGTH];
    TCHAR  fmtSpec[MAXSTRLENGTH];
    
    mbstowcs (fmtSpec, (const char *)str, MAXSTRLENGTH);
    wsprintf (buffer, fmtSpec, arg);
    RETAILMSG(1, (TEXT("%s"), buffer));
}


/** ============================================================================
 *  @func   MESSAGEMULTIDSP_1Print
 *
 *  @desc   Print a message with two arguments.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_2Print (Char8 * str, Uint32 arg, Uint32 arg1)
{
    TCHAR  buffer[MAXSTRLENGTH];
    TCHAR  fmtSpec[MAXSTRLENGTH];
    
    mbstowcs (fmtSpec, (const char *)str, MAXSTRLENGTH);
    wsprintf (buffer, fmtSpec, arg, arg1);
    RETAILMSG(1, (TEXT("%s"), buffer));
}



/** ============================================================================
 *  @func   MESSAGEMULTIDSP_Sleep
 *
 *  @desc   Sleeps for the specified number of microseconds.
 *
 *  @modif  None
 *  ============================================================================
 */
NORMAL_API
Void
MESSAGEMULTIDSP_Sleep (Uint32 uSec)
{
    Sleep ((uSec + 999)/1000) ;   
}


#if defined (MESSAGE_PROFILE)

/** ============================================================================
 *  @func   MESSAGE_GetStartTime ()
 *
 *  @desc   This function records the reset time in microseconds.
 *
 *  @modif  startUsecTime
 *  ============================================================================
 */
EXPORT_API
Void
MESSAGEMULTIDSP_GetStartTime (Void)
{
    startMsecTime = GetTickCount();   
}


/** ============================================================================
 *  @func   MESSAGE_GetEndTime ()
 *
 *  @desc   This function records the reset time in microseconds.
 *
 *  @modif  endUsecTime
 *  ============================================================================
 */
EXPORT_API
Void
MESSAGEMULTIDSP_GetEndTime (Void)
{
    endMsecTime = GetTickCount();        
}

/** ============================================================================
 *  @func   MESSAGE_GetProfileInfo
 *
 *  @desc   It converts a time variable to elapsed time structure.
 *
 *  @modif  None
 *  ============================================================================
 */
EXPORT_API
Void
MESSAGEMULTIDSP_GetProfileInfo (Uint32 numIterations)
{    
    Real32     roundTripTime = 0    ;
    Uint32     msecTimeTaken = 0    ;
    Real32     numSeconds    = 0    ;

    msecTimeTaken = (endMsecTime - startMsecTime) ;
    if (msecTimeTaken != 0) {
        roundTripTime = (Real32)(  (Real32)(msecTimeTaken)
                                 / (Real32)(numIterations)) ;
    }

    /* To calculate number of seconds in msecTimeTaken */
    numSeconds      = (Real32)((Real32) msecTimeTaken / 1000.0) ;

    MESSAGE_1Print ("Transferring %d iterations took ", numIterations) ;
    MESSAGE_1Print ("%d seconds ", numSeconds) ;
    MESSAGE_1Print ("%d milliseconds.\n", (Real32)(msecTimeTaken)) ;

    MESSAGE_1Print ("RoundTrip Time for 1 message is %d milliseconds.\n",
                    roundTripTime) ;

    return ;
    
}

#endif /* if defined (MESSAGE_PROFILE) */

#if defined (__cplusplus)
}
#endif /* defined (__cplusplus) */
